from speculator.features import sma
import unittest

class SMATest(unittest.TestCase):
    def test_eval_algorithm(self):
        closes = [11, 12, 13, 14, 15, 16, 17]
        self.assertEqual(sma.eval_algorithm(closes), 14)




from speculator.utils import date
from speculator.utils import poloniex
from speculator.utils import stats

"""
Simple Moving Average:
Average closing price over a period
SMA = avg(closes) = sum(closes) / len(closes)
"""

def eval_algorithm(closes):
    """ Evaluates the SMA algorithm
    
    Args:
        closes: List of price closes.

    Returns:
        Float average of closes.
    """
    return stats.avg(closes)

def get_poloniex(*args):
    """ Gets SMA of a currency pair from Poloniex.com exchange

    Returns:
        Float average of closes.
    """
    return from_poloniex(poloniex.get_json_shift(*args))

def from_poloniex(json):
    """ Gets SMA from a JSON of market data
    
    Args:
        json: List of dates where each entry is a dict of raw market data.
    
    Returns:
        Float average of closes.
    """
    closes = [date['close'] for date in json]
    return eval_algorithm(closes)



import logging
import requests
from speculator.utils import date

logging.basicConfig(level=logging.INFO)
logger = logging.getLogger(__name__)

def json_to_url(json, symbol):
    """ Converts a JSON to a URL by the Poloniex API 

    Args:
        json: JSON data as a list of dict dates, where the keys are
            the raw market statistics.
        symbol: String of currency pair, like a ticker symbol.

    Returns:
        String URL to Poloniex API representing the given JSON.
    """
    start = json[0]['date']
    end = json[-1]['date']
    diff = end - start

    # Get period by a ratio from calculated period to valid periods
    # Ratio closest to 1 is the period
    # Valid values: 300, 900, 1800, 7200, 14400, 86400
    periods = [300, 900, 1800, 7200, 14400, 86400]

    diffs = {}
    for p in periods:
        diffs[p] = abs(1 - (p / (diff / len(json)))) # Get ratio

    period = min(diffs, key=diffs.get) # Find closest period
    
    url = ('https://poloniex.com/public?command'
           '=returnChartData&currencyPair={0}&start={1}'
           '&end={2}&period={3}').format(symbol, start, end, period) 
    return url

def chart_json(start, end, period, symbol):
    """ Requests chart data from Poloniex API
    
    Args:
        start: Int epoch date to START getting market stats from.
            Note that this epoch is FURTHER from the current date.
        end: Int epoch date to STOP getting market stats from.
            Note that this epoch is CLOSER to the current date.
        period: Int defining width of each chart candlestick in seconds.
            Valid values: 300, 900, 1800, 7200, 14400, 86400
        symbol: String of currency pair, like a ticker symbol.

    Returns:
        Tuple of (JSON data, URL to JSON).
        JSON data as a list of dict dates, where the keys are
        the raw market statistics.
        String URL to Poloniex API representing the given JSON.
    """
    url = ('https://poloniex.com/public?command'
           '=returnChartData&currencyPair={0}&start={1}'
           '&end={2}&period={3}').format(symbol, start, end, period) 
    logger.debug(' HTTP Request URL:\n{0}'.format(url))
    json = requests.get(url).json()
    logger.debug(' JSON:\n{0}'.format(json))

    if 'error' in json:
        logger.error(' Invalid parameters in URL for HTTP response')
        raise SystemExit
    elif all(val == 0 for val in json[0]):
        logger.error(' Bad HTTP response.  Time unit too short?')
        raise SystemExit
    elif len(json) < 1: # time to short
        logger.error(' Not enough dates to calculate changes')
        raise SystemExit

    return json, url

def parse_changes(json):
    """ Gets price changes from JSON

    Args:
        json: JSON data as a list of dict dates, where the keys are
            the raw market statistics.

    Returns:
        List of floats of price changes between entries in JSON.
    """
    changes = []
    dates = len(json)
    for date in range(1, dates): 
        last_close = json[date - 1]['close']
        now_close = json[date]['close']
        changes.append(now_close - last_close)
    logger.debug('Market Changes (from JSON):\n{0}'.format(changes))
    return changes

def get_gains_losses(changes):
    """ Categorizes changes into gains and losses

    Args:
        changes: List of floats of price changes between entries in JSON.

    Returns:
        Dict of changes with keys 'gains' and 'losses'.
        All values are positive.
    """
    res = {'gains': [], 'losses': []}
    for change in changes:
        if change > 0:
            res['gains'].append(change)
        else:
            res['losses'].append(change * -1)
    logger.debug('Gains: {0}'.format(res['gains']))
    logger.debug('Losses: {0}'.format(res['losses']))
    return res

def get_attribute(json, attr):
    """ Gets the values of an attribute from JSON

    Args:
        json: JSON data as a list of dict dates, where the keys are
            the raw market statistics.
        attr: String of attribute in JSON file to collect.

    Returns:
        List of values of specified attribute from JSON
    """
    res = [json[entry][attr] for entry, _ in enumerate(json)]
    logger.debug('{0}s (from JSON):\n{1}'.format(attr, res))
    return res

def get_json_shift(year, month, day, unit, count, period, symbol):
    """ Gets JSON from shifted date by the Poloniex API

    Args:
        year: Int between 1 and 9999.
        month: Int between 1 and 12.
        day: Int between 1 and 31.
        unit: String of time period unit for count argument.
            How far back to check historical market data.
            Valid values: 'hour', 'day', 'week', 'month', 'year'
        count: Int of units.
            How far back to check historical market data.
        period: Int defining width of each chart candlestick in seconds.
        symbol: String of currency pair, like a ticker symbol.

    Returns: JSON, list of dates where each entry is a dict of raw market data.
    """
    epochs = date.get_end_start_epochs(year, month, day, 'last', unit, count)
    return chart_json(epochs['shifted'], epochs['initial'],
                      period, symbol)[0]




from speculator.utils import date
from speculator.utils import poloniex
from speculator.utils import stats

"""
Relative Strength Index:
RSI = 100 - (100 / (1 + RS))
    such that,
        RS = avg(t-period gain) / avg(t-period loss)
"""

def eval_algorithm(gains, losses):
    """ Evaluates the RSI algorithm
    
    Args:
        gains: List of price gains.
        losses: List of prices losses.

    Returns:
        Float between 0 and 100, momentum indicator
        of a market measuring the speed and change of price movements.
    """
    return 100 - (100 / (1 + eval_rs(gains, losses)))

def eval_rs(gains, losses):
    """ Evaluates the RS variable in RSI algorithm

    Args:
        gains: List of price gains.
        losses: List of prices losses.

    Returns:
        Float of average gains over average losses.
    """
    # Number of days that the data was collected through
    count = len(gains) + len(losses)

    avg_gains = stats.avg(gains, count=count)
    avg_losses = stats.avg(losses,count=count)

    try:
        return avg_gains / avg_losses
    except ZeroDivisionError: # No losses
        return avg_gains

def get_poloniex(*args):
    """ Gets RSI of a currency pair from Poloniex.com exchange

    Returns:
        Float between 0 and 100, momentum indicator
        of a market measuring the speed and change of price movements.
    """
    return from_poloniex(poloniex.get_json_shift(*args))

def from_poloniex(json):
    """ Gets RSI from a JSON of market data
    
    Args:
        json: List of dates where each entry is a dict of raw market data.
    
    Returns:
        Float between 0 and 100, momentum indicator
        of a market measuring the speed and change of price movements.
    """
    changes = poloniex.get_gains_losses(poloniex.parse_changes(json))
    return eval_algorithm(changes['gains'], changes['losses'])



"""
Various math functions used throughout Speculator
"""

def avg(vals, count=None):
    """ Returns the average value

    Args:
        vals: List of numbers to calculate average from.
        count: Int of total count that vals was part of.

    Returns:
        Float average value throughout a count.
    """ 
    sum = 0
    for v in vals:
        sum += v
    if count is None:
        count = len(vals)
    return float(sum) / count




from speculator.utils import date
import unittest

class DateTest(unittest.TestCase):
    def test_shift_epoch(self):
        delorean = date.date_to_delorean(2000, 1, 1)
        shift = date.shift_epoch(delorean, 'last', 'day', 2)
        self.assertEqual(shift, 946512000)

    def test_generate_epochs(self):
        delorean = date.date_to_delorean(2000, 1, 1)
        gen = date.generate_epochs(delorean, 'last', 'day', 3)
        epochs = [e for e in gen]
        
        expected_epochs = [946684800, 946598400, 946512000]
        self.assertEqual(epochs, expected_epochs) 
        



from datetime import datetime as dt
from delorean import Delorean

def date_to_delorean(year, month, day):
    """ Converts date arguments to a Delorean instance in UTC
    
    Args:
        year: int between 1 and 9999.
        month: int between 1 and 12.
        day: int between 1 and 31.

    Returns:
        Delorean instance in UTC of date.
    """
    return Delorean(datetime=dt(year, month, day), timezone='UTC')

def date_to_epoch(year, month, day):
    """ Converts a date to epoch in UTC

    Args:
        year: int between 1 and 9999.
        month: int between 1 and 12.
        day: int between 1 and 31.

    Returns:
        Int epoch in UTC from date.
    """
    return int(date_to_delorean(year, month, day).epoch)

def now_delorean():
    """ Returns the current time as a Delorean instance in UTC """
    return Delorean(timezone='UTC')

def shift_epoch(delorean, direction, unit, count):
    """ Gets the resulting epoch after a time shift of a Delorean
    
    Args:
        delorean: Delorean datetime instance to shift from.
        direction: String to shift time forwards or backwards.
            Valid values: 'last', 'next'.
        unit: String of time period unit for count argument.
            What unit in direction should be shifted?
            Valid values: 'hour', 'day', 'week', 'month', 'year'.
        count: Int of units.
            How many units to shift in direction?

    Returns:
        Int epoch in UTC from a shifted Delorean
    """
    return int(delorean._shift_date(direction, unit, count).epoch)

def generate_epochs(delorean, direction, unit, count):
    """ Generates epochs from a shifted Delorean instance
    
    Args:
        delorean: Delorean datetime instance to shift from.
        direction: String to shift time forwards or backwards.
            Valid values: 'last', 'next'.
        unit: String of time period unit for count argument.
            What unit in direction should be shifted?
            Valid values: 'hour', 'day', 'week', 'month', 'year'.
        count: Int of units.
            How many units to shift in direction?

    Returns:
        Generator of count int epochs in UTC from a shifted Delorean
    """
    for shift in range(count):
        yield int(delorean._shift_date(direction, unit, shift).epoch)

def get_end_start_epochs(year, month, day, direction, unit, count):
    """ Gets epoch from a start date and epoch from a shifted date

    Args:
        year: Int between 1 and 9999.
        month: Int between 1 and 12.
        day: Int between 1 and 31.
        direction: String to shift time forwards or backwards.
            Valid values: 'last', 'next'.
        unit: String of time period unit for count argument.
            How far back to check historical market data.
            Valid values: 'hour', 'day', 'week', 'month', 'year'.
        count: Int of units.
            How far back to check historical market data?

    Returns:
        Dict of int epochs in UTC with keys 'initial' and 'shifted'
    """
    if year or month or day: # Date is specified
        if not year:
            year = 2017
        if not month:
            month = 1
        if not day:
            day = 1
        initial_delorean = date_to_delorean(year, month, day)
    else: # Date is not specified, get current date
        count += 1 # Get another date because market is still open
        initial_delorean = now_delorean()
    
    initial_epoch = int(initial_delorean.epoch)
    shifted_epoch = shift_epoch(initial_delorean, direction, unit, count)
    return { 'initial': initial_epoch, 'shifted': shifted_epoch }




from speculator.utils import date
from speculator.utils import poloniex

"""
Stochastic Oscillator:
%K = 100 * (C - L(t)) / (H14 - L(t))
    such that,
        C = Current closing Price
        L(t) = Lowest Low over some duration t
        H(t) = Highest High over some duration t
          * t is usually 14 days

%K follows the speed/momentum of a price in a market
"""

def eval_algorithm(closing, low, high):
    """ Evaluates the SO algorithm

    Args:
        closing: Float of current closing price.
        low: Float of lowest low closing price throughout some duration.
        high: Float of highest high closing price throughout some duration.

    Returns:
        Float SO between 0 and 100.
    """
    try:
        return 100 * (closing - low) / (high - low)
    except ZeroDivisionError: # High and low are the same
        return 100 * (closing - low)

def get_poloniex(*args):
    """ Gets SO of a currency pair from Poloniex.com exchange

    Returns:
        Float SO between 0 and 100.
    """
    return from_poloniex(poloniex.get_json_shift(*args))

def from_poloniex(json):
    """ Gets SO from a JSON of market data
    
    Args:
        json: List of dates where each entry is a dict of raw market data.
    
    Returns:
        Float SO between 0 and 100.
    """
    close  = json[-1]['close'] # Latest closing price
    low    = min(poloniex.get_attribute(json, 'low')) # Lowest low
    high   = max(poloniex.get_attribute(json, 'high')) # Highest high
    return eval_algorithm(close, low, high)




from speculator.features import rsi
from speculator.features import sma
from speculator.features import so
from speculator.utils import date
from speculator.utils import poloniex
import unittest
        
# https://poloniex.com/public?command=returnChartData&currencyPair=USDT_BTC&start=1483228800&end=1483315200&period=86400

EXPECTED_RESPONSE = [{'close': 999.36463982,
                      'date': 1483228800,
                      'high': 1008.54999326,
                      'low': 957.02,
                      'open': 965.00000055,
                      'quoteVolume': 1207.33863593,
                      'volume': 1196868.2615889,
                      'weightedAverage': 991.32772361},
                     {'close': 1019.00000076,
                      'date': 1483315200,
                      'high': 1034.32896003,
                      'low': 994.00000044,
                      'open': 999.92218873,
                      'quoteVolume': 1818.58703006,
                      'volume': 1847781.3863449,
                      'weightedAverage': 1016.0533182}]

EXPECTED_CHANGES = ([EXPECTED_RESPONSE[1]['close'] -
                     EXPECTED_RESPONSE[0]['close']])

YEAR          = 2017
MONTH         = 1
DAY           = 1
UNIT          = 'day'
COUNT         = 3
PERIOD        = 86400
SYMBOL        = 'USDT_BTC'
EPOCH1        = 1483228800 # 01/01/2017, 00:00 epoch
EPOCH2        = 1483315200 # 01/02/2017, 00:00 epoch
HTTP_RESPONSE = poloniex.chart_json(EPOCH1, EPOCH2, PERIOD, SYMBOL)[0]

class PoloniexIntegrationTest(unittest.TestCase):
    def test_parse_changes(self):
        self.assertEqual(poloniex.parse_changes(HTTP_RESPONSE),
                         EXPECTED_CHANGES) 

    def test_get_gains_losses(self):
        res = {'gains': [g for g in EXPECTED_CHANGES if g >= 0],
               'losses': [l for l in EXPECTED_CHANGES if l < 0]}
        self.assertEqual(poloniex.get_gains_losses(
                         poloniex.parse_changes(HTTP_RESPONSE)), res)

    def test_get_attribute(self):
        res = [attribute['quoteVolume'] for attribute in HTTP_RESPONSE]
        attr = 'quoteVolume'
        self.assertEqual(poloniex.get_attribute(HTTP_RESPONSE, attr), res) 

    def test_get_rsi_poloniex(self):
        eval_rsi = rsi.get_poloniex(YEAR, MONTH, DAY,
                                    UNIT, COUNT, PERIOD, SYMBOL) 
        self.assertAlmostEqual(eval_rsi, 71.888274, places=4)

    def test_get_stoch_osc_poloniex(self):
        eval_so = so.get_poloniex(YEAR, MONTH, DAY,
                                  UNIT, COUNT, PERIOD, SYMBOL)
        self.assertAlmostEqual(eval_so, 88.9532721773, places=4)

    def test_get_sma_poloniex(self):
        eval_sma = sma.get_poloniex(YEAR, MONTH, DAY,
                                    UNIT, COUNT, PERIOD, SYMBOL)
        self.assertAlmostEqual(eval_sma, 974.808582, places=4)




import argparse
from datetime import datetime as dt
import pandas as pd
from speculator import market
from speculator import models

"""
Predicts the next market trend based on the markets history
"""

def get_args():
    today = dt.now()
    parser = argparse.ArgumentParser(
                        description='Predict the next crypto market trend.')
    parser.add_argument('-m', '--model', default='random_forest',
                        help=('Machine learning model to use on market data'))
    parser.add_argument('-u', '--unit', default='month',
                        help=('Time unit to go back: '
                              'second, minute, hour, day, week, month, year'))
    parser.add_argument('-c', '--count', default=6, type=int,
                        help=('Number of "units" to go back '
                              '(behind the set date/now)'))
    parser.add_argument('-pd', '--period', default=86400, type=int,
                        help=('Number of seconds for each candlestick: '
                              '300, 900, 1800, 7200, 14400, 86400 only valid'))
    parser.add_argument('-pn', '--partition', default=14, type=int,
                        help=('Number of candlesticks per '
                              'technical analysis calculation'))
    parser.add_argument('-sy', '--symbol', default='USDT_BTC',
                        help=('Currency pair, symbol, or ticker, from Poloniex'
                              'Examples: USDT_BTC, ETH_ZRX, BTC_XRP'))
    parser.add_argument('-sd', '--seed', default=None, type=int,
                        help=('Random state seed to be used when generating '
                              'the Random Forest and training/test sets'))
    parser.add_argument('-l', '--long', nargs='*',
                        help=('List of features to enable longer calculations. '
                              'Example "--long rsi": 14 day RSI -> 28 day RSI'))
    parser.add_argument('-d', '--delta', default=25, type=int,
                        help=('Price buffer in the neutral zone before'
                              'classifying a trend as bullish or bearish'))
    parser.add_argument('-t', '--trees', default=10, type=int,
                        help=('Number of trees (estimators) to generate in '
                              'the Random Forest, higher is usually better'))
    parser.add_argument('-j', '--jobs', default=1, type=int,
                        help=('Number of jobs (CPUs for multithreading) when '
                              'processing model fits and predictions'))
    parser.add_argument('--proba', default=False, action='store_true',
                        help='Display probabilities of the predicted trend')
    parser.add_argument('--proba_log', default=False, action='store_true',
                        help=('Display logarithmic probabilities of the '
                              'predicted trend'))
    return parser.parse_args()

def predict(model_type, x, y=None):
    if model_type == 'random_forest' or model_type == 'rf':
        pass
    elif model_type == 'deep_neural_network' or model_type == 'dnn':
        pass

def main():
    args = get_args()

    m = market.Market(symbol=args.symbol, unit=args.unit,
                      count=args.count, period=args.period)
    x = m.features(partition=args.partition)
    if args.long is not None:
        # Create long features DataFrame
        x_long = m.features(partition=2 * args.partition)

        # Remove features not specified by args.long
        unwanted_features = [f for f in x.columns if f not in args.long]
        x_long = x_long.drop(unwanted_features, axis=1)
        
        # Prefix long columns with 'long_' to fix naming conflicts
        x_long.columns = ['long_{0}'.format(f) for f in x_long.columns]

        # Merge the two DataFrames
        skip = args.partition
        x = pd.concat([x[skip:].reset_index(drop=True), x_long], axis=1)

    y = market.targets(x, delta=args.delta)
    x = x.drop(['close'], axis=1)
    model = market.setup_model(x[:-1], y,
                               model_type=args.model.lower(),
                               seed=args.seed,
                               n_estimators=args.trees,
                               n_jobs=args.jobs)

    next_date = x.tail(1) # Remember the entry we didn't train?  Predict it.

    # TODO: Reimplement display of confusion matrix and feature importances
    acc = model.accuracy(model.features.test, model.targets.test)
    print('Test Set Accuracy: {0:.3f}%'.format(100 * acc))

    trends = model._predict_trends(next_date)
    print('Predicted Trend: {0}'.format(market.target_code_to_name(trends[0])))

    if args.proba:
        probas = model._predict_probas(next_date)
        print('Probability: {0}'.format(probas[0]))
    if args.proba_log:
        logs = model._predict_logs(next_date)
        print('Log Probability: {0}'.format(logs[0]))

if __name__=='__main__':
    raise SystemExit(main())



from speculator.features import rsi
import unittest

class RSITest(unittest.TestCase):
    def test_eval_rs(self):
        gains = [0.07, 0.73, 0.51, 0.28, 0.34, 0.43, 0.25, 0.15, 0.68, 0.24]
        losses = [0.23, 0.53, 0.18, 0.40]
        self.assertAlmostEqual(rsi.eval_rs(gains, losses), 2.746, places=3)

    def test_eval_algorithm(self):
        gains = [0.07, 0.73, 0.51, 0.28, 0.34, 0.43, 0.25, 0.15, 0.68, 0.24]
        losses = [0.23, 0.53, 0.18, 0.40]
        self.assertAlmostEqual(rsi.eval_algorithm(gains, losses),
                               73.307, places=3)
        



from speculator.utils import stats
import unittest

class StatsTest(unittest.TestCase):
    def test_avg(self):
        nums = [0.24, 0.62, 0.15, 0.83, 0.12345] 
        self.assertEqual(stats.avg(nums), 0.39269)

        nums = [0.24, 0.62, 0.15, 0.83, 0.12345] 
        self.assertEqual(stats.avg(nums, count=8), 0.24543125)
